/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.yoko.bindings.corba.extensions;

import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.ExtensionDeserializer;
import javax.wsdl.extensions.ExtensionRegistry;
import javax.xml.namespace.QName;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

import org.apache.schemas.yoko.bindings.corba.BindingType;
import org.apache.yoko.wsdl.CorbaConstants;

public class CorbaBindingExtensionHelper implements ExtensionDeserializer {

    public CorbaBindingExtensionHelper() {
    }

    public static void addExtension(ExtensionRegistry registry,
            Class parentType, String element) {
        QName extQName = new QName(CorbaConstants.NU_WSDL_CORBA, element, CorbaConstants.NU_WSDL_CORBA);
        CorbaBindingExtensionHelper helper = new CorbaBindingExtensionHelper();

        registry.registerDeserializer(parentType, extQName, helper);
    }

    public ExtensibilityElement unmarshall(Class parentType, QName elementType,
            Element el, Definition def, ExtensionRegistry extReg)
        throws WSDLException {

        BindingType bindType = new BindingType();
        bindType.setElementType(elementType);

        // Store the repository ids of the interface and bases
        NamedNodeMap bindAttributes = el.getAttributes();
        for (int i = 0; i < bindAttributes.getLength(); ++i) {
            if (bindAttributes.item(i).getNodeName().equals("repositoryID")) {
                bindType.setRepositoryID(bindAttributes.item(i).getNodeValue());
            } else if (bindAttributes.item(i).getNodeName().equals("bases")) {
                String allBases = bindAttributes.item(i).getNodeValue();
                String[] bases = allBases.split(" ");
                for (int j = 0; j < bases.length; ++j) {
                    bindType.getBases().add(bases[j]);
                }
            }
        }

        return bindType;
    }
}
