/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.yoko.bindings.corba.extensions;

import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.ExtensionDeserializer;
import javax.wsdl.extensions.ExtensionRegistry;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.apache.schemas.yoko.bindings.corba.ArgType;
import org.apache.schemas.yoko.bindings.corba.ModeType;
import org.apache.schemas.yoko.bindings.corba.OperationType;
import org.apache.schemas.yoko.bindings.corba.ParamType;
import org.apache.schemas.yoko.bindings.corba.RaisesType;

import org.apache.yoko.wsdl.CorbaConstants;

public class CorbaOperationExtensionHelper implements ExtensionDeserializer {

    public CorbaOperationExtensionHelper() {
    }

    public static void addExtension(ExtensionRegistry registry,
            Class parentType, String element) {
        QName extQName = new QName(CorbaConstants.NU_WSDL_CORBA, element, CorbaConstants.NP_WSDL_CORBA);
        CorbaOperationExtensionHelper helper = new CorbaOperationExtensionHelper();

        registry.registerDeserializer(parentType, extQName, helper);
    }

    public ExtensibilityElement unmarshall(Class parentType, QName elementType,
            Element el, Definition def, ExtensionRegistry extReg)
        throws WSDLException {

        OperationType opType = new OperationType();

        // Store the operation name
        NamedNodeMap opAttributes = el.getAttributes();
        for (int i = 0; i < opAttributes.getLength(); ++i) {
            if (opAttributes.item(i).getNodeName().equals("name")) {
                opType.setName(opAttributes.item(i).getNodeValue());
            }
        }

        // Store the operations parameters, return value, and raises
        NodeList opChildNodes = el.getChildNodes();
        for (int i = 0; i < opChildNodes.getLength(); ++i) {
            Node currentNode = opChildNodes.item(i);

            if (currentNode.getNodeName().equals("corba:param")) {
                ParamType param = new ParamType();
                NamedNodeMap paramAttributes = currentNode.getAttributes();

                for (int j = 0; j < paramAttributes.getLength(); ++j) {
                    Node paramAttrNode = paramAttributes.item(j);
                    if (paramAttrNode.getNodeName().equals("name")) {
                        param.setName(paramAttrNode.getNodeValue());
                    } else if (paramAttrNode.getNodeName().equals("mode")) {
                        param.setMode(ModeType.fromValue(paramAttrNode.getNodeValue()));
                    } else if (paramAttrNode.getNodeName().equals("idltype")) {
                        param.setIdltype(getIdlTypeName(def, paramAttrNode, currentNode));
                    }
                }
                opType.getParam().add(param);
            } else if (currentNode.getNodeName().equals("corba:return")) {
                ArgType ret = new ArgType();
                NamedNodeMap retAttributes = currentNode.getAttributes();

                for (int j = 0; j < retAttributes.getLength(); ++j) {
                    Node retAttrNode = retAttributes.item(j);
                    if (retAttrNode.getNodeName().equals("idltype")) {
                        ret.setIdltype(getIdlTypeName(def, retAttrNode, currentNode));
                    }
                }
                opType.setReturn(ret);
            } else if (currentNode.getNodeName().equals("corba:raises")) {
                RaisesType raises = new RaisesType();
                NamedNodeMap raiseAttributes = currentNode.getAttributes();

                for (int j = 0; j < raiseAttributes.getLength(); ++j) {
                    Node raiseAttrNode = raiseAttributes.item(j);
                    if (raiseAttrNode.getNodeName().equals("exception")) {
                        raises.setException(getIdlTypeName(def, raiseAttrNode, currentNode));
                    }
                }
                opType.getRaises().add(raises);
            }
            // anything else we are not interested in at the moment
        }
        return opType;
    }

    private QName getIdlTypeName(Definition def, Node node, Node parent) {
        String type = node.getNodeValue();
        int seperatorIndex = type.indexOf(':');
        String prefix = null;
        if (seperatorIndex != -1) {
            prefix = type.substring(0, seperatorIndex);
        }
        String localPart = type.substring(seperatorIndex + 1, type.length());
        String nsURI = parent.lookupNamespaceURI(prefix);
        return new QName(nsURI, localPart);
    }
}
