/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.xml.namespace.QName;

import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.Sequence;
import org.apache.schemas.yoko.bindings.corba.TypeMappingType;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.apache.ws.commons.schema.XmlSchemaComplexType;
import org.apache.ws.commons.schema.XmlSchemaElement;
import org.apache.ws.commons.schema.XmlSchemaSequence;
import org.apache.ws.commons.schema.XmlSchemaType;

import org.apache.yoko.wsdl.CorbaConstants;

public class ArrayVisitor extends TypesVisitorBase {

    public ArrayVisitor(XmlSchemaCollection xmlSchemas,
                        XmlSchema xmlSchema,
                        TypeMappingType typeMapRef) {
        super(xmlSchemas, xmlSchema, typeMapRef);
    }
    
    public void visit(AST node) {
        AST seq = node.getFirstChild();
        AST node2 = seq.getFirstChild();
        XmlSchemaType stype = TypesUtils.findType(schemas, schema, node2);
        if (stype == null) {
            //System.err.println("Couldn't find type: " + node2);
        } else {
            long bound = -1;
            if (node2.getNextSibling() != null) {
                bound = Long.parseLong(node2.getNextSibling().toString());
            }
            node2 = seq.getNextSibling();
            XmlSchemaComplexType ct = new XmlSchemaComplexType(getSchema());
            String name = node2.toString();
            ct.setName(name);
            XmlSchemaSequence sequence = new XmlSchemaSequence();
            XmlSchemaElement el = new XmlSchemaElement();
            el.setName("item");
            el.setMinOccurs(0);
            if (bound != -1) {
                el.setMaxOccurs(bound);
            } else {
                el.setMaxOccurs(Long.MAX_VALUE);
            }
            el.setSchemaTypeName(stype.getQName());
            sequence.getItems().add(el);
            ct.setParticle(sequence);
            getSchema().getItems().add(ct);
            getSchema().addType(ct);

            //create the corba sequence
            Sequence corbaSeq = new Sequence();
            if (bound == -1) {
                bound = 0;
            }                
            corbaSeq.setBound(bound);
            corbaSeq.setQName(new QName(typeMap.getTargetNamespace(), name));
            corbaSeq.setType(ct.getQName());
            corbaSeq.setElemtype(TypesUtils.findCorbaType(getCorbaTypeMap(), stype.getQName()).getQName());
            corbaSeq.setRepositoryID(CorbaConstants.REPO_STRING
                                     + name
                                     + CorbaConstants.IDL_VERSION);
            getCorbaTypeMap().getStructOrExceptionOrUnion().add(corbaSeq);
            setSchemaType(ct);
            setCorbaType(corbaSeq);
        }
    }

}
