/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.xml.namespace.QName;

import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.MemberType;
import org.apache.schemas.yoko.bindings.corba.Struct;
import org.apache.schemas.yoko.bindings.corba.TypeMappingType;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.apache.ws.commons.schema.XmlSchemaComplexType;
import org.apache.ws.commons.schema.XmlSchemaElement;
import org.apache.ws.commons.schema.XmlSchemaSequence;
import org.apache.ws.commons.schema.XmlSchemaType;

import org.apache.yoko.wsdl.CorbaConstants;

public class StructVisitor extends TypesVisitorBase {
    
    public StructVisitor(XmlSchemaCollection xmlSchemas,
                         XmlSchema xmlSchema,
                         TypeMappingType typeMappingType) {
        super(xmlSchemas, xmlSchema, typeMappingType);
    }

    public void visit(AST node) {
        AST structNode = node.getFirstChild();
        
        // xmlschema:struct
        String structName = structNode.toString();
        XmlSchemaComplexType complexType = new XmlSchemaComplexType(schema);
        complexType.setName(structName);
        XmlSchemaSequence sequence = new XmlSchemaSequence();
        complexType.setParticle(sequence);

        
        // corba:struct
        Struct struct = new Struct();
        struct.setQName(new QName(typeMap.getTargetNamespace(), structName));
        struct.setType(complexType.getQName());
        struct.setRepositoryID(CorbaConstants.REPO_STRING
                               + structName
                               + CorbaConstants.IDL_VERSION);

        
        // struct members
        AST memberTypeNode = structNode.getNextSibling();
        while (memberTypeNode != null) {
            AST memberNode = memberTypeNode.getNextSibling();

            // xmlschema:member
            XmlSchemaElement member = new XmlSchemaElement();
            String memberName = memberNode.toString();
            member.setName(memberName);
            // type
            XmlSchemaType type = TypesUtils.findType(schemas, schema, memberTypeNode);
            member.setSchemaType(type);
            member.setSchemaTypeName(type.getQName());

            sequence.getItems().add(member);

            
            // corba:member
            MemberType memberType = new MemberType();
            memberType.setName(memberName);
            QName idlType = TypesUtils.findCorbaType(typeMap, type.getQName()).getQName();
            memberType.setIdltype(idlType);
            struct.getMember().add(memberType);
            
            
            memberTypeNode = memberNode.getNextSibling();
        }


        // add struct to schema
        schema.getItems().add(complexType);
        schema.addType(complexType);

        // add struct to corba typemap
        typeMap.getStructOrExceptionOrUnion().add(struct);
        
        setSchemaType(complexType);
        setCorbaType(struct);
    }
    
}
