/*
 *  Copyright 2006 The Apache Software Foundation or its licensors, as applicable.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.yoko.orb.OB;

import org.apache.yoko.orb.OB.WLogger;

public class Logger_impl extends org.omg.CORBA.LocalObject implements WLogger {
    // ------------------------------------------------------------------
    // Logger_impl private member implementations
    // ------------------------------------------------------------------

    private void message(String msg) {
        synchronized (System.err) {
            System.err.println(msg);
        }
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public void info(String msg) {
        message(msg);
    }

    public void winfo(String msg) {
        info(msg);
    }

    public void error(String msg) {
        message("error: " + msg);
    }

    public void werror(String msg) {
        error(msg);
    }

    public void warning(String msg) {
        message("warning: " + msg);
    }

    public void wwarning(String msg) {
        warning(msg);
    }

    public void trace(String category, String msg) {
        String s = "[ " + category + ": ";
        int start = 0;
        int next;
        while ((next = msg.indexOf('\n', start)) != -1) {
            s += msg.substring(start, next + 1);
            s += "  ";
            start = next + 1;
        }
        s += msg.substring(start);
        s += " ]";
        message(s);
    }

    public void wtrace(String category, String msg) {
        trace(category, msg);
    }
}
